From e48fb9c68ec898711a0a3bc7fb765406e814caa0 Mon Sep 17 00:00:00 2001
From: Mark Dilger <mark.dilger@enterprisedb.com>
Date: Sun, 26 Jan 2020 10:41:37 -0800
Subject: [PATCH 2/2] Adding frontend tests for json parser.

Creating tests in new directory src/test/bin for testing
that the json parser can be included and used from within
a frontend standalone binary.
---
 src/Makefile                    |  4 ++-
 src/test/Makefile               |  6 ++--
 src/test/bin/.gitignore         |  1 +
 src/test/bin/Makefile           | 41 ++++++++++++++++++++++
 src/test/bin/README             | 11 ++++++
 src/test/bin/t/001_test_json.pl | 47 +++++++++++++++++++++++++
 src/test/bin/test_json.c        | 62 +++++++++++++++++++++++++++++++++
 7 files changed, 168 insertions(+), 4 deletions(-)
 create mode 100644 src/test/bin/.gitignore
 create mode 100644 src/test/bin/Makefile
 create mode 100644 src/test/bin/README
 create mode 100644 src/test/bin/t/001_test_json.pl
 create mode 100644 src/test/bin/test_json.c

diff --git a/src/Makefile b/src/Makefile
index bcdbd9588a..ccd4bab0de 100644
--- a/src/Makefile
+++ b/src/Makefile
@@ -2,7 +2,8 @@
 #
 # Makefile for src
 #
-# Copyright (c) 1994, Regents of the University of California
+# Portions Copyright (c) 1994, Regents of the University of California
+# Portions Copyright (c) 1996-2020, PostgreSQL Global Development Group
 #
 # src/Makefile
 #
@@ -27,6 +28,7 @@ SUBDIRS = \
 	bin \
 	pl \
 	makefiles \
+	test/bin \
 	test/regress \
 	test/isolation \
 	test/perl
diff --git a/src/test/Makefile b/src/test/Makefile
index efb206aa75..6c3a1d4c27 100644
--- a/src/test/Makefile
+++ b/src/test/Makefile
@@ -12,7 +12,7 @@ subdir = src/test
 top_builddir = ../..
 include $(top_builddir)/src/Makefile.global
 
-SUBDIRS = perl regress isolation modules authentication recovery subscription
+SUBDIRS = bin perl regress isolation modules authentication recovery subscription
 
 # Test suites that are not safe by default but can be run if selected
 # by the user via the whitespace-separated list in variable
@@ -40,10 +40,10 @@ endif
 ALWAYS_SUBDIRS = $(filter-out $(SUBDIRS),examples kerberos ldap locale thread ssl)
 
 # We want to recurse to all subdirs for all standard targets, except that
-# installcheck and install should not recurse into the subdirectory "modules".
+# installcheck and install should not recurse into "modules" or "bin"
 
 recurse_alldirs_targets := $(filter-out installcheck install, $(standard_targets))
-installable_dirs := $(filter-out modules, $(SUBDIRS))
+installable_dirs := $(filter-out modules bin, $(SUBDIRS))
 
 $(call recurse,$(recurse_alldirs_targets))
 $(call recurse,installcheck, $(installable_dirs))
diff --git a/src/test/bin/.gitignore b/src/test/bin/.gitignore
new file mode 100644
index 0000000000..6709c749d8
--- /dev/null
+++ b/src/test/bin/.gitignore
@@ -0,0 +1 @@
+test_json
diff --git a/src/test/bin/Makefile b/src/test/bin/Makefile
new file mode 100644
index 0000000000..3eee9091bc
--- /dev/null
+++ b/src/test/bin/Makefile
@@ -0,0 +1,41 @@
+#-------------------------------------------------------------------------
+#
+# Makefile for src/test/bin
+#
+# Portions Copyright (c) 1996-2020, PostgreSQL Global Development Group
+# Portions Copyright (c) 1994, Regents of the University of California
+#
+# src/bin/bin/Makefile
+#
+#-------------------------------------------------------------------------
+
+PGFILEDESC = "bin - the PostgreSQL standalone code binaries for testing"
+PGAPPICON=win32
+
+subdir = src/test/bin
+top_builddir = ../../..
+include $(top_builddir)/src/Makefile.global
+
+# make this available to TAP test scripts
+export with_readline
+
+REFDOCDIR= $(top_srcdir)/doc/src/sgml/ref
+
+override CPPFLAGS := -I. -I$(srcdir) -I$(libpq_srcdir) $(CPPFLAGS)
+LDFLAGS_INTERNAL += -L$(top_builddir)/src/fe_utils -lpgfeutils $(libpq_pgport)
+
+OBJS = \
+	$(WIN32RES) \
+	test_json.o
+
+all: test_json
+
+test_json: $(OBJS) | submake-libpq submake-libpgport submake-libpgfeutils
+	$(CC) $(CFLAGS) $(OBJS) $(LDFLAGS) $(LDFLAGS_EX) $(LIBS) -o $@$(X)
+
+check:
+	PATH="$(abs_top_builddir)/src/test/bin:$$PATH" $(prove_check)
+
+clean distclean maintainer-clean:
+	rm -f test_json$(X) $(OBJS)
+	rm -rf tmp_check
diff --git a/src/test/bin/README b/src/test/bin/README
new file mode 100644
index 0000000000..239bd8456f
--- /dev/null
+++ b/src/test/bin/README
@@ -0,0 +1,11 @@
+src/test/bin/README
+
+Binary executable tests
+=======================
+
+This directory contains programs that are built and executed for testing
+purposes, but never installed. It may be used, for example, to test that code
+in src/common works in frontend environments.
+
+These test programs are part of the tap-test suite.  Configure with tap tests
+enabled or these tests will be skipped.
diff --git a/src/test/bin/t/001_test_json.pl b/src/test/bin/t/001_test_json.pl
new file mode 100644
index 0000000000..a848fd66cc
--- /dev/null
+++ b/src/test/bin/t/001_test_json.pl
@@ -0,0 +1,47 @@
+# Basic logical replication test
+use strict;
+use warnings;
+use TestLib ();
+use Cwd;
+
+use Test::More tests => 83;
+
+# There doesn't seem to be any easy way to get TestLib to use the binaries from
+# our directory, so we hack up a path to our binary and run that directly.  This
+# seems brittle enough that some other solution should be found, if possible.
+
+my $test_json = join('/', $ENV{TESTDIR}, 'test_json');
+
+ok(-f $test_json, "test_json file exists");
+ok(-x $test_json, "test_json file is executable");
+
+# Verify some valid JSON is accepted by our parser
+TestLib::command_like( [$test_json, q/null/       ], qr{\bVALID\b}, "null");
+TestLib::command_like( [$test_json, q/{}/         ], qr{\bVALID\b}, "empty object");
+TestLib::command_like( [$test_json, q/[]/         ], qr{\bVALID\b}, "empty array");
+TestLib::command_like( [$test_json, q/-12345/     ], qr{\bVALID\b}, "negative integer");
+TestLib::command_like( [$test_json, q/-1/         ], qr{\bVALID\b}, "negative integer");
+TestLib::command_like( [$test_json, q/0/          ], qr{\bVALID\b}, "zero");
+TestLib::command_like( [$test_json, q/1/          ], qr{\bVALID\b}, "positive integer");
+TestLib::command_like( [$test_json, q/12345/      ], qr{\bVALID\b}, "positive integer");
+TestLib::command_like( [$test_json, q/-1.23456789/], qr{\bVALID\b}, "negative float");
+TestLib::command_like( [$test_json, q/1.23456789/ ], qr{\bVALID\b}, "positive float");
+TestLib::command_like( [$test_json, q/{"a": "b"}/ ], qr{\bVALID\b}, "object");
+TestLib::command_like( [$test_json, q/["a", "b"]/ ], qr{\bVALID\b}, "array");
+TestLib::command_like( [$test_json, q/"pigs feet"/], qr{\bVALID\b}, 'text string');
+
+# Verify some invalid JSON is rejected by our parser
+TestLib::command_like( [$test_json, q/{/          ], qr{^\s*The input string ended unexpectedly\.\s*$}ms, 'unclosed object');
+TestLib::command_like( [$test_json, q/[/          ], qr{^\s*The input string ended unexpectedly\.\s*$}ms, 'unclosed array');
+TestLib::command_like( [$test_json, q/(/          ], qr{^\s*Token "\(" is invalid\.\s*$}ms, 'unclosed parenthesis');
+TestLib::command_like( [$test_json, q/}/          ], qr{^\s*Expected JSON value, but found "\}"\.\s*$}ms, 'unopened object');
+TestLib::command_like( [$test_json, q/]/          ], qr{^\s*Expected JSON value, but found "\]"\.\s*$}ms, 'unopened array');
+TestLib::command_like( [$test_json, q/)/          ], qr{^\s*Token "\)" is invalid\.\s*$}ms, 'unopened parenthesis');
+TestLib::command_like( [$test_json, q/{{{}}/      ], qr{^\s*Expected string or "\}", but found "\{"\.\s*$}ms, 'unbalanced object curlies');
+TestLib::command_like( [$test_json, q/{{}}}/      ], qr{^\s*Expected string or "\}", but found "\{"\.\s*$}ms, 'unbalanced object curlies');
+TestLib::command_like( [$test_json, q/[[[]]/      ], qr{^\s*The input string ended unexpectedly\.\s*$}ms, 'unbalanced array braces');
+TestLib::command_like( [$test_json, q/[[]]]/      ], qr{^\s*Expected end of input, but found "\]"\.\s*$}ms, 'unbalanced array braces');
+TestLib::command_like( [$test_json, q/((())/      ], qr{^\s*Token "\(" is invalid\.\s*$}ms, 'unbalanced array braces');
+TestLib::command_like( [$test_json, q/(()))/      ], qr{^\s*Token "\(" is invalid\.\s*$}ms, 'unbalanced array braces');
+TestLib::command_like( [$test_json, q/1 7 13/     ], qr{^\s*Expected end of input, but found "7"\.\s*$}ms, 'integer sequence');
+TestLib::command_like( [$test_json, q/{"a", "b"}/ ], qr{^\s*Expected ":", but found ","\.\s*$}ms, 'mixed object and array syntax');
diff --git a/src/test/bin/test_json.c b/src/test/bin/test_json.c
new file mode 100644
index 0000000000..86ad9e7e2b
--- /dev/null
+++ b/src/test/bin/test_json.c
@@ -0,0 +1,62 @@
+/*
+ *	pg_test_json.c
+ *		tests validity of json strings against parser implementation.
+ */
+
+#include "postgres_fe.h"
+
+#include "common/jsonapi.h"
+#include "libpq-fe.h"
+
+static const char *progname;
+
+static void parse_json(const char *str);
+
+int
+main(int argc, char *argv[])
+{
+	int			argidx;
+
+	progname = get_progname(argv[0]);
+
+	if (argc > 1)
+	{
+		if (strcmp(argv[1], "--help") == 0 || strcmp(argv[1], "-?") == 0)
+		{
+			printf(_("Usage: %s jsonstr [, ...]\n"), progname);
+			exit(0);
+		}
+		if (strcmp(argv[1], "--version") == 0 || strcmp(argv[1], "-V") == 0)
+		{
+			puts("pg_test_json (PostgreSQL) " PG_VERSION);
+			exit(0);
+		}
+	}
+
+	for (argidx = 1; argidx < argc; argidx++)
+		parse_json(argv[argidx]);
+
+	return 0;
+}
+
+static void
+parse_json(const char *str)
+{
+	char *json;
+	JsonLexContext *lex;
+	int client_encoding;
+	JsonParseErrorType parse_result;
+
+	client_encoding = PQenv2encoding();
+
+	json = strdup(str);
+	lex = makeJsonLexContextCstringLen(json, strlen(json), client_encoding, true /* need_escapes */);
+	parse_result = pg_parse_json(lex, &nullSemAction);
+	if (JSON_SUCCESS == parse_result)
+		fprintf(stdout, _("VALID\n"));
+	else
+	{
+		const char *errstr = json_errdetail(parse_result, lex);
+		fprintf(stdout, _("%s\n"), errstr);
+	}
+}
-- 
2.21.1 (Apple Git-122.3)

